from dataclasses import dataclass
from typing import List, Optional
from edgescan.data.types.object import Object

import datetime


@dataclass(frozen=True)
class Vulnerability(Object):
    altered_score: bool
    asset_id: int
    asset_name: str
    confidence: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    cves: List[str]
    cvss_score: Optional[float]
    cvss_vector: Optional[str]
    cvss_version: Optional[str]
    cvss_v2_score: Optional[float]
    cvss_v2_vector: Optional[str]
    date_opened: datetime.datetime
    date_closed: datetime.datetime
    definition_id: int
    id: int
    label: Optional[str]
    last_pci_exception: Optional[str]
    layer: str
    location: str
    location_specifier_id: Optional[int]
    name: str
    pci_compliance_status: str
    risk: int
    severity: int
    status: str
    threat: int

    @property
    def create_time(self) -> datetime.datetime:
        return self.created_at

    @property
    def update_time(self) -> datetime.datetime:
        return self.updated_at

    @property
    def open_time(self) -> datetime.datetime:
        return self.date_opened

    @property
    def close_time(self) -> datetime.datetime:
        return self.date_closed

    def affects_pci_compliance(self) -> bool:
        return self.pci_compliance_status == 'fail'

    def is_network_layer_vulnerability(self) -> bool:
        return self.layer == 'network'

    def is_application_layer_vulnerability(self) -> bool:
        return self.layer == 'application'



from typing import Optional

from edgescan.constants import ASSETS, VULNERABILITIES, HOSTS
from edgescan.data.types.assessment import Assessment
from edgescan.data.types.asset import Asset
from edgescan.data.types.host import Host
from edgescan.data.types.license import License
from edgescan.data.types.location_specifier import LocationSpecifier
from edgescan.data.types.object import Object
from edgescan.data.types.vulnerability import Vulnerability

import hodgepodge.time


def parse_object(data: dict, collection_type: str) -> Object:
    parser = _PARSERS_BY_COLLECTION_TYPE[collection_type]
    return parser(data)


def parse_asset(data: dict) -> Asset:
    data = _parse_timestamps(data)
    data.update({
        'active_license': parse_license(data.pop('active_licence')),
        'last_host_scan': hodgepodge.time.to_datetime(data['last_host_scan']),
        'location_specifiers': [LocationSpecifier(**loc) for loc in data['location_specifiers']],
    })
    assessment = data['current_assessment']
    if assessment:
        data['current_assessment'] = parse_assessment(assessment)
    return Asset(**data)


def parse_vulnerability(data: dict) -> Vulnerability:
    data = _parse_timestamps(data)
    return Vulnerability(**data)


def parse_assessment(data: dict) -> Assessment:
    data = _parse_timestamps(data)
    return Assessment(**data)


def parse_license(data: dict) -> License:
    data = _parse_timestamps(data)
    data.update({
        'license_type_id': data.pop('licence_type_id'),
        'license_type_name': data.pop('licence_type_name'),
    })
    return License(**data)


def parse_host(data: dict) -> Host:
    data = _parse_timestamps(data)
    return Host(**data)


def parse_location_specifier(data: dict) -> LocationSpecifier:
    return LocationSpecifier(**data)


def _parse_timestamps(data: Optional[dict]) -> Optional[dict]:
    if data:
        for k, v in data.items():
            if k.startswith('date_') or k.endswith(('_at', '_date')):
                data[k] = hodgepodge.time.to_datetime(v)
    return data


_PARSERS_BY_COLLECTION_TYPE = {
    ASSETS: parse_asset,
    VULNERABILITIES: parse_vulnerability,
    HOSTS: parse_host,
}



from dataclasses import dataclass
from typing import List, Any, Optional
from edgescan.data.types.assessment import Assessment
from edgescan.data.types.license import License
from edgescan.data.types.location_specifier import LocationSpecifier
from edgescan.data.types.object import Object

import datetime


@dataclass(frozen=True)
class Asset(Object):
    asset_status: str
    authenticated: bool
    active_license: License
    blocked_status: str
    created_at: datetime.datetime
    current_assessment: Assessment
    host_count: int
    hostname: str
    id: int
    last_assessment_date: datetime.datetime
    last_host_scan: datetime.datetime
    linked_assets: List[Any]
    location_specifiers: List[LocationSpecifier]
    name: str
    network_access: str
    next_assessment_date: Optional[datetime.datetime]
    pci_enabled: Optional[bool]
    priority: int
    tags: List[str]
    targeting_mode: str
    type: str
    updated_at: datetime.datetime

    @property
    def create_time(self):
        return self.created_at

    @property
    def update_time(self):
        return self.updated_at



from dataclasses import dataclass
from edgescan.data.types.object import Object

import datetime


@dataclass(frozen=True)
class Assessment(Object):
    id: int
    type: str
    status: str
    created_at: datetime.datetime

    @property
    def create_time(self) -> datetime.datetime:
        return self.created_at



from edgescan.errors import MissingCredentialsError

import os

DEFAULT_API_KEY = os.getenv('EDGESCAN_API_KEY')


def validate_api_key(key: str) -> str:
    if not key:
        raise MissingCredentialsError("An API key is required")
    return key



from edgescan.api.client import EdgeScan

import itertools
import json
import edgescan.cli.click as click


@click.group()
@click.pass_context
def assets(_):
    """
    Query or count assets.
    """
    pass


@assets.command()
@click.option('--id', 'asset_id', type=int, required=True)
@click.pass_context
def get_asset(ctx: click.Context, asset_id: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    asset = api.get_asset(asset_id)
    if asset:
        click.echo(asset.to_json())


@assets.command()
@click.option('--ids')
@click.option('--names')
@click.option('--tags')
@click.option('--limit', type=int)
@click.pass_context
def get_assets(ctx: click.Context, ids: str, names: str, tags: str, limit: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    for asset in api.iter_assets(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        tags=click.str_to_strs(tags),
        limit=limit,
    ):
        click.echo(asset.to_json())


@assets.command()
@click.option('--ids')
@click.option('--names')
@click.option('--tags')
@click.pass_context
def get_asset_tags(ctx: click.Context, ids: str, names: str, tags: str):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    rows = api.iter_assets(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        tags=click.str_to_strs(tags),
    )
    tags = sorted(set(itertools.chain.from_iterable(row.tags for row in rows)))
    click.echo(json.dumps(tags))


@assets.command()
@click.option('--ids')
@click.option('--names')
@click.option('--tags')
@click.pass_context
def count_assets(ctx: click.Context, ids: str, names: str, tags: str):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    total = api.count_assets(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        tags=click.str_to_strs(tags),
    )
    click.echo(total)



from edgescan.api.authentication import DEFAULT_API_KEY
from edgescan.api.host import DEFAULT_HOST
from edgescan.cli.command_groups.assets import assets
from edgescan.cli.command_groups.hosts import hosts
from edgescan.cli.command_groups.licenses import licenses
from edgescan.cli.command_groups.vulnerabilities import vulnerabilities

import edgescan.cli.click as click
import os

_env = os.environ


@click.group()
@click.option('--host', default=DEFAULT_HOST, help='${{EDGESCAN_HOST}} {}'.format('✔' if 'EDGESCAN_HOST' in _env else '✖'))
@click.option('--api-key', default=DEFAULT_API_KEY, help='${{EDGESCAN_API_KEY}} {}'.format('✔' if 'EDGESCAN_API_KEY' in _env else '✖'))
@click.pass_context
def cli(ctx: click.Context, host: str, api_key: str):
    ctx.ensure_object(dict)
    ctx.obj.update({
        'config': {
            'edgescan': {
                'api': {
                    'host': host,
                    'api_key': api_key,
                }
            }
        }
    })


COMMAND_GROUPS = [
    assets,
    hosts,
    licenses,
    vulnerabilities,
]
for command_group in COMMAND_GROUPS:
    cli.add_command(command_group)



from typing import List, Optional
from click import *

import hodgepodge.click
import hodgepodge.time
import datetime


def str_to_strs(data: str) -> List[str]:
    return hodgepodge.click.str_to_strs(data)


def str_to_ints(data: str) -> List[int]:
    return hodgepodge.click.str_to_ints(data)


def str_to_datetime(data: str) -> Optional[datetime.datetime]:
    if data:
        return hodgepodge.time.to_datetime(data)



from dataclasses import dataclass
from typing import Optional, List
from edgescan.data.types.object import Object

import hodgepodge.platforms
import datetime


@dataclass(frozen=True)
class Host(Object):
    id: int
    asset_id: int
    location: str
    hostnames: List[str]
    label: Optional[str]
    status: str
    updated_at: datetime.datetime
    os_name: str
    apis_detected: bool

    @property
    def os_type(self) -> str:
        return hodgepodge.platforms.parse_os_type(self.os_name)

    @property
    def os_version(self) -> str:
        return self.os_version

    @property
    def update_time(self) -> datetime.datetime:
        return self.updated_at

    def is_alive(self) -> bool:
        return self.status == 'alive'



from dataclasses import dataclass
from edgescan.constants import DEFAULT_INDENT, SORT_KEYS_BY_DEFAULT, REMOVE_EMPTY_VALUES_BY_DEFAULT

import hodgepodge.types


@dataclass(frozen=True)
class Object:
    id: int

    def to_dict(self, remove_empty_values: bool = REMOVE_EMPTY_VALUES_BY_DEFAULT):
        return hodgepodge.types.dataclass_to_dict(data=self, remove_empty_values=remove_empty_values)

    def to_json(
            self,
            indent: int = DEFAULT_INDENT,
            sort_keys: bool = SORT_KEYS_BY_DEFAULT,
            remove_empty_values: bool = REMOVE_EMPTY_VALUES_BY_DEFAULT):

        data = self.to_dict(remove_empty_values=remove_empty_values)
        return hodgepodge.types.dict_to_json(
            data=data,
            indent=indent,
            sort_keys=sort_keys,
            remove_empty_values=remove_empty_values,
        )



from typing import Optional
from edgescan.api.client import EdgeScan

import edgescan.cli.click as click


@click.group()
@click.pass_context
def hosts(_):
    """
    Query or count hosts.
    """
    pass


@hosts.command()
@click.option('--id', 'host_id', type=int, required=True)
@click.pass_context
def get_host(ctx: click.Context, host_id: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    host = api.get_host(host_id)
    if host:
        click.echo(host.to_json())


@hosts.command()
@click.option('--ids')
@click.option('--hostnames')
@click.option('--asset-ids')
@click.option('--asset-tags')
@click.option('--ip-addresses')
@click.option('--os-types')
@click.option('--os-versions')
@click.option('--alive/--dead', default=None)
@click.option('--limit', type=int)
@click.pass_context
def get_hosts(
        ctx: click.Context,
        ids: str,
        hostnames: str,
        asset_ids: str,
        asset_tags: str,
        ip_addresses: str,
        os_types: str,
        os_versions: str,
        alive: Optional[bool],
        limit: int):

    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    for host in api.iter_hosts(
        ids=click.str_to_ints(ids),
        hostnames=click.str_to_strs(hostnames),
        asset_ids=click.str_to_ints(asset_ids),
        asset_tags=click.str_to_strs(asset_tags),
        ip_addresses=click.str_to_strs(ip_addresses),
        os_types=click.str_to_strs(os_types),
        os_versions=click.str_to_strs(os_versions),
        alive=alive,
        limit=limit,
    ):
        click.echo(host.to_json())


@hosts.command()
@click.option('--ids')
@click.option('--hostnames')
@click.option('--asset-ids')
@click.option('--asset-tags')
@click.option('--ip-addresses')
@click.option('--os-types')
@click.option('--os-versions')
@click.option('--alive/--dead', default=None)
@click.pass_context
def count_hosts(
        ctx: click.Context,
        ids: str,
        hostnames: str,
        asset_ids: str,
        asset_tags: str,
        ip_addresses: str,
        os_types: str,
        os_versions: str,
        alive: Optional[bool]):

    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    total = api.count_hosts(
        ids=click.str_to_ints(ids),
        hostnames=click.str_to_strs(hostnames),
        asset_ids=click.str_to_ints(asset_ids),
        asset_tags=click.str_to_strs(asset_tags),
        ip_addresses=click.str_to_strs(ip_addresses),
        os_types=click.str_to_strs(os_types),
        os_versions=click.str_to_strs(os_versions),
        alive=alive,
    )
    click.echo(total)



from typing import Optional
from edgescan.api.client import EdgeScan

import edgescan.cli.click as click


@click.group()
@click.pass_context
def vulnerabilities(_):
    """
    Query or count vulnerabilities.
    """
    pass


@vulnerabilities.command()
@click.option('--id', 'vulnerability_id', type=int, required=True)
@click.pass_context
def get_vulnerability(ctx: click.Context, vulnerability_id: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    vulnerability = api.get_vulnerability(vulnerability_id)
    if vulnerability:
        click.echo(vulnerability.to_json())


@vulnerabilities.command()
@click.option('--ids')
@click.option('--names')
@click.option('--cve-ids')
@click.option('--asset-ids')
@click.option('--asset-tags')
@click.option('--ip-addresses')
@click.option('--affects-pci-compliance/--does-not-affect-pci-compliance', default=None)
@click.option('--include-application-layer-vulnerabilities/--exclude-application-layer-vulnerabilities', default=True)
@click.option('--include-network-layer-vulnerabilities/--exclude-network-layer-vulnerabilities', default=True)
@click.option('--limit', type=int)
@click.pass_context
def get_vulnerabilities(
        ctx: click.Context,
        ids: Optional[str],
        names: Optional[str],
        cve_ids: Optional[str],
        asset_ids: Optional[str],
        asset_tags: Optional[str],
        ip_addresses: Optional[str],
        affects_pci_compliance: Optional[bool],
        include_application_layer_vulnerabilities: Optional[bool],
        include_network_layer_vulnerabilities: Optional[bool],
        limit: int):

    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    for vulnerability in api.iter_vulnerabilities(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        cve_ids=click.str_to_strs(cve_ids),
        asset_ids=click.str_to_ints(asset_ids),
        asset_tags=click.str_to_strs(asset_tags),
        ip_addresses=click.str_to_strs(ip_addresses),
        affects_pci_compliance=affects_pci_compliance,
        include_application_layer_vulnerabilities=include_application_layer_vulnerabilities,
        include_network_layer_vulnerabilities=include_network_layer_vulnerabilities,
        limit=limit,
    ):
        click.echo(vulnerability.to_json())


@vulnerabilities.command()
@click.option('--ids')
@click.option('--names')
@click.option('--cve-ids')
@click.option('--asset-ids')
@click.option('--asset-tags')
@click.option('--ip-addresses')
@click.option('--affects-pci-compliance/--does-not-affect-pci-compliance', default=None)
@click.option('--include-application-layer-vulnerabilities/--exclude-application-layer-vulnerabilities', default=True)
@click.option('--include-network-layer-vulnerabilities/--exclude-network-layer-vulnerabilities', default=True)
@click.pass_context
def count_vulnerabilities(
        ctx: click.Context,
        ids: Optional[str],
        names: Optional[str],
        cve_ids: Optional[str],
        asset_ids: Optional[str],
        asset_tags: Optional[str],
        ip_addresses: Optional[str],
        affects_pci_compliance: Optional[bool],
        include_application_layer_vulnerabilities: Optional[bool],
        include_network_layer_vulnerabilities: Optional[bool]):

    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    total = api.count_vulnerabilities(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        cve_ids=click.str_to_strs(cve_ids),
        asset_ids=click.str_to_ints(asset_ids),
        asset_tags=click.str_to_strs(asset_tags),
        ip_addresses=click.str_to_strs(ip_addresses),
        affects_pci_compliance=affects_pci_compliance,
        include_application_layer_vulnerabilities=include_application_layer_vulnerabilities,
        include_network_layer_vulnerabilities=include_network_layer_vulnerabilities,
    )
    click.echo(total)



from edgescan.api.authentication import DEFAULT_API_KEY
from hodgepodge.http import DEFAULT_MAX_RETRIES_ON_CONNECTION_ERRORS, DEFAULT_MAX_RETRIES_ON_READ_ERRORS, \
    DEFAULT_MAX_RETRIES_ON_REDIRECT, DEFAULT_BACKOFF_FACTOR, Session

import hodgepodge.http
import edgescan.api.authentication


def get_session(
        api_key: str = DEFAULT_API_KEY,
        max_retries_on_connection_errors: int = DEFAULT_MAX_RETRIES_ON_CONNECTION_ERRORS,
        max_retries_on_read_errors: int = DEFAULT_MAX_RETRIES_ON_READ_ERRORS,
        max_retries_on_redirects: int = DEFAULT_MAX_RETRIES_ON_REDIRECT,
        backoff_factor: float = DEFAULT_BACKOFF_FACTOR) -> Session:

    session = Session()
    session.headers.update({
        'X-Api-Token': edgescan.api.authentication.validate_api_key(api_key),
        'Content-Type': 'application/json'
    })
    auto_retry_policy = hodgepodge.http.get_automatic_retry_policy(
        max_retries_on_connection_errors=max_retries_on_connection_errors,
        max_retries_on_read_errors=max_retries_on_read_errors,
        max_retries_on_redirects=max_retries_on_redirects,
        backoff_factor=backoff_factor,
    )
    hodgepodge.http.attach_session_policies(session=session, policies=[auto_retry_policy])
    return session



from typing import List, Iterator, Any, Optional
from edgescan.api.authentication import DEFAULT_API_KEY
from edgescan.api.host import DEFAULT_HOST
from edgescan.constants import COLLECTION_TYPES
from edgescan.data.types.asset import Asset
from edgescan.data.types.host import Host
from edgescan.data.types.license import License
from edgescan.data.types.vulnerability import Vulnerability
from edgescan.http.session import DEFAULT_MAX_RETRIES_ON_CONNECTION_ERRORS, DEFAULT_MAX_RETRIES_ON_READ_ERRORS, \
    DEFAULT_MAX_RETRIES_ON_REDIRECT, DEFAULT_BACKOFF_FACTOR

import edgescan.data.parser as parser
import edgescan.http.session
import hodgepodge.pattern_matching
import hodgepodge.platforms
import urllib.parse
import logging

logging.basicConfig(level=logging.INFO)

logger = logging.getLogger(__name__)


class EdgeScan:
    def __init__(
            self,
            host: str = DEFAULT_HOST,
            api_key: str = DEFAULT_API_KEY,
            max_retries_on_connection_errors: int = DEFAULT_MAX_RETRIES_ON_CONNECTION_ERRORS,
            max_retries_on_read_errors: int = DEFAULT_MAX_RETRIES_ON_READ_ERRORS,
            max_retries_on_redirects: int = DEFAULT_MAX_RETRIES_ON_REDIRECT,
            backoff_factor: float = DEFAULT_BACKOFF_FACTOR):

        self.host = host
        self.url = 'https://{}'.format(host)
        self.session = edgescan.http.session.get_session(
            api_key=api_key,
            max_retries_on_connection_errors=max_retries_on_connection_errors,
            max_retries_on_read_errors=max_retries_on_read_errors,
            max_retries_on_redirects=max_retries_on_redirects,
            backoff_factor=backoff_factor,
        )

    @property
    def hosts_url(self) -> str:
        return urllib.parse.urljoin(self.url, 'api/v1/hosts.json')

    @property
    def assets_url(self) -> str:
        return urllib.parse.urljoin(self.url, 'api/v1/assets.json')

    @property
    def vulnerabilities_url(self) -> str:
        return urllib.parse.urljoin(self.url, 'api/v1/vulnerabilities.json')

    def _iter_objects(self, url: str) -> Iterator[Any]:
        collection_type = _get_collection_type_from_url(url)

        response = self.session.get(url)
        response.raise_for_status()

        reply = response.json()
        for row in reply[collection_type]:
            row = parser.parse_object(row, collection_type=collection_type)
            if row:
                yield row

    def _count_objects(self, url: str) -> int:
        response = self.session.get(url)
        response.raise_for_status()

        reply = response.json()
        return reply['total']

    def get_host(self, host_id: int) -> Optional[Host]:
        return next(self.iter_hosts(ids=[host_id]), None)

    def get_hosts(
            self,
            ids: Optional[List[int]] = None,
            hostnames: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            os_types: Optional[List[str]] = None,
            os_versions: Optional[List[str]] = None,
            alive: Optional[bool] = None,
            limit: Optional[int] = None) -> List[Host]:

        return list(self.iter_hosts(
            ids=ids,
            hostnames=hostnames,
            asset_ids=asset_ids,
            asset_tags=asset_tags,
            ip_addresses=ip_addresses,
            os_types=os_types,
            os_versions=os_versions,
            alive=alive,
            limit=limit,
        ))

    def iter_hosts(
            self,
            ids: Optional[List[int]] = None,
            hostnames: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            os_types: Optional[List[str]] = None,
            os_versions: Optional[List[str]] = None,
            alive: Optional[bool] = None,
            limit: Optional[int] = None) -> Iterator[Host]:

        os_types = None if not os_types else [hodgepodge.platforms.parse_os_type(os_type) for os_type in os_types]

        #: If we're filtering hosts by asset tag, we'll first need to lookup assets and filter hosts by asset tag.
        if asset_tags:
            assets = self.iter_assets(ids=asset_ids, tags=asset_tags)
            asset_ids = {asset.id for asset in assets}

        i = 0
        for host in self._iter_objects(url=self.hosts_url):
            if ids and host.id not in ids:
                continue

            if os_types and not hodgepodge.pattern_matching.str_matches_glob(host.hostnames, hostnames):
                continue

            if asset_ids and host.asset_id not in asset_ids:
                continue

            if ip_addresses and host.location not in ip_addresses:
                continue

            if os_types and not hodgepodge.pattern_matching.str_matches_glob(host.os_type, os_types):
                continue

            if os_versions and not hodgepodge.pattern_matching.str_matches_glob(host.os_version, os_versions):
                continue

            if alive is not None and alive != host.is_alive():
                continue

            yield host

            if limit:
                i += 1
                if i >= limit:
                    return

    def count_hosts(
            self,
            ids: Optional[List[int]] = None,
            hostnames: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            os_types: Optional[List[str]] = None,
            os_versions: Optional[List[str]] = None,
            alive: Optional[bool] = None) -> int:

        hosts = self.iter_hosts(
            ids=ids,
            hostnames=hostnames,
            asset_ids=asset_ids,
            asset_tags=asset_tags,
            ip_addresses=ip_addresses,
            os_types=os_types,
            os_versions=os_versions,
            alive=alive,
        )
        return sum(1 for _ in hosts)

    def get_asset(self, asset_id: int) -> Optional[Asset]:
        return next(self.iter_assets(ids=[asset_id]), None)

    def get_assets(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            tags: Optional[List[str]] = None,
            limit: Optional[int] = None) -> List[Asset]:

        return list(self.iter_assets(
            ids=ids,
            names=names,
            tags=tags,
            limit=limit,
        ))

    def iter_assets(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            tags: Optional[List[str]] = None,
            limit: Optional[int] = None) -> Iterator[Asset]:

        tags = set(tags) if tags else None

        i = 0
        for asset in self._iter_objects(url=self.assets_url):
            if ids and asset.id not in ids:
                continue

            if names and not hodgepodge.pattern_matching.str_matches_glob(asset.name, names):
                continue

            if tags and set(asset.tags).isdisjoint(tags):
                continue

            yield asset

            if limit:
                i += 1
                if i >= limit:
                    return

    def count_assets(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            tags: Optional[List[str]] = None) -> int:

        assets = self.iter_assets(
            ids=ids,
            names=names,
            tags=tags,
        )
        return sum(1 for _ in assets)

    def get_vulnerability(self, vulnerability_id: int) -> Optional[Vulnerability]:
        return next(self.iter_vulnerabilities(ids=[vulnerability_id]), None)

    def get_vulnerabilities(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            cve_ids: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            affects_pci_compliance: Optional[bool] = None,
            include_application_layer_vulnerabilities: Optional[bool] = True,
            include_network_layer_vulnerabilities: Optional[bool] = True,
            limit: Optional[int] = None) -> List[Vulnerability]:

        return list(self.iter_vulnerabilities(
            ids=ids,
            names=names,
            cve_ids=cve_ids,
            asset_ids=asset_ids,
            asset_tags=asset_tags,
            ip_addresses=ip_addresses,
            affects_pci_compliance=affects_pci_compliance,
            include_application_layer_vulnerabilities=include_application_layer_vulnerabilities,
            include_network_layer_vulnerabilities=include_network_layer_vulnerabilities,
            limit=limit,
        ))

    def iter_vulnerabilities(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            cve_ids: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            affects_pci_compliance: Optional[bool] = None,
            include_application_layer_vulnerabilities: Optional[bool] = None,
            include_network_layer_vulnerabilities: Optional[bool] = None,
            limit: Optional[int] = None) -> Iterator[Vulnerability]:

        if asset_tags:
            assets = self.iter_assets(ids=asset_ids, tags=asset_tags)
            asset_ids = {asset.id for asset in assets}

        i = 0
        for vulnerability in self._iter_objects(url=self.vulnerabilities_url):
            if ids and vulnerability.id not in ids:
                continue

            if names and not hodgepodge.pattern_matching.str_matches_glob(vulnerability.name, names):
                continue

            if asset_ids and vulnerability.asset_id not in asset_ids:
                continue

            if cve_ids and not hodgepodge.pattern_matching.str_matches_glob(vulnerability.cves, cve_ids):
                continue

            if ip_addresses and not hodgepodge.pattern_matching.str_matches_glob(vulnerability.location, ip_addresses):
                continue

            if affects_pci_compliance is not None and affects_pci_compliance != vulnerability.affects_pci_compliance():
                continue

            if include_application_layer_vulnerabilities is False and vulnerability.is_application_layer_vulnerability():
                continue

            if include_network_layer_vulnerabilities is False and vulnerability.is_network_layer_vulnerability():
                continue

            yield vulnerability

            if limit:
                i += 1
                if i >= limit:
                    return

    def count_vulnerabilities(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            cve_ids: Optional[List[str]] = None,
            asset_ids: Optional[List[int]] = None,
            asset_tags: Optional[List[str]] = None,
            ip_addresses: Optional[List[str]] = None,
            affects_pci_compliance: Optional[bool] = None,
            include_application_layer_vulnerabilities: Optional[bool] = None,
            include_network_layer_vulnerabilities: Optional[bool] = None) -> int:

        vulnerabilities = self.iter_vulnerabilities(
            ids=ids,
            names=names,
            cve_ids=cve_ids,
            asset_ids=asset_ids,
            asset_tags=asset_tags,
            ip_addresses=ip_addresses,
            affects_pci_compliance=affects_pci_compliance,
            include_application_layer_vulnerabilities=include_application_layer_vulnerabilities,
            include_network_layer_vulnerabilities=include_network_layer_vulnerabilities,
        )
        return sum(1 for _ in vulnerabilities)

    def get_license(self, license_id: int) -> Optional[License]:
        return next(self.iter_licenses(ids=[license_id]), None)

    def get_licenses(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            expired: Optional[bool] = None,
            limit: Optional[int] = None) -> List[License]:

        return list(self.iter_licenses(
            ids=ids,
            names=names,
            expired=expired,
            limit=limit,
        ))

    def iter_licenses(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            expired: Optional[bool] = None,
            limit: Optional[int] = None) -> Iterator[License]:

        i = 0
        for row in self._iter_licenses():
            if ids and row.id not in ids:
                continue

            if names and not hodgepodge.pattern_matching.str_matches_glob(row.name, names):
                continue

            if expired is not None and row.expired != expired:
                continue

            yield row

            if limit:
                i += 1
                if i >= limit:
                    return

    def _iter_licenses(self) -> Iterator[License]:
        seen = set()
        for asset in self.iter_assets():
            active_license = asset.active_license
            if active_license.id not in seen:
                seen.add(active_license.id)
                yield active_license

    def count_licenses(
            self,
            ids: Optional[List[int]] = None,
            names: Optional[List[str]] = None,
            expired: Optional[bool] = None) -> int:

        licenses = self.iter_licenses(
            ids=ids,
            names=names,
            expired=expired,
        )
        return sum(1 for _ in licenses)


def _get_collection_type_from_url(url: str) -> str:
    v = urllib.parse.urlsplit(url).path.split('/')[-1]
    if v.endswith('.json'):
        v = v[:-5]

    if v not in COLLECTION_TYPES:
        raise ValueError("Failed to parse collection type from URL: {} - got '{}' - allowed: {}".format(
            url, v, COLLECTION_TYPES
        ))
    return v



from typing import Optional
from dataclasses import dataclass
from edgescan.data.types.object import Object

import datetime


@dataclass(frozen=True)
class License(Object):
    id: int
    name: str
    license_type_id: int
    license_type_name: str
    asset_id: Optional[int]
    order_id: int
    start_date: datetime.datetime
    end_date: datetime.datetime
    expired: bool
    status: Optional[str]

    def is_expired(self):
        return self.expired



from edgescan.api.client import EdgeScan

import edgescan.cli.click as click


@click.group()
@click.pass_context
def licenses(_):
    """
    Query or count licenses.
    """
    pass


@licenses.command()
@click.option('--id', 'license_id', type=int, required=True)
@click.pass_context
def get_license(ctx: click.Context, license_id: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    row = api.get_license(license_id)
    if row:
        click.echo(row.to_json())


@licenses.command()
@click.option('--ids')
@click.option('--names')
@click.option('--expired/--not-expired', default=None)
@click.option('--limit', type=int)
@click.pass_context
def get_licenses(ctx: click.Context, ids: str, names: str, expired: bool, limit: int):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    for row in api.iter_licenses(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        expired=expired,
        limit=limit,
    ):
        click.echo(row.to_json())


@licenses.command()
@click.option('--ids')
@click.option('--names')
@click.option('--expired/--not-expired', default=None)
@click.pass_context
def count_licenses(ctx: click.Context, ids: str, names: str, expired: bool):
    api = EdgeScan(**ctx.obj['config']['edgescan']['api'])
    total = api.count_licenses(
        ids=click.str_to_ints(ids),
        names=click.str_to_strs(names),
        expired=expired,
    )
    click.echo(total)



DEFAULT_INDENT = 4
SORT_KEYS_BY_DEFAULT = True
REMOVE_EMPTY_VALUES_BY_DEFAULT = False

#: Collection types.
HOSTS = 'hosts'
ASSETS = 'assets'
VULNERABILITIES = 'vulnerabilities'
COLLECTION_TYPES = [HOSTS, ASSETS, VULNERABILITIES]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['edgescan',
 'edgescan.api',
 'edgescan.cli',
 'edgescan.cli.command_groups',
 'edgescan.data',
 'edgescan.data.types',
 'edgescan.http']

package_data = \
{'': ['*']}

install_requires = \
['hodgepodge>=2.3.3,<3.0.0']

entry_points = \
{'console_scripts': ['edgescan = edgescan.cli:cli']}

setup_kwargs = {
    'name': 'edgescan',
    'version': '0.1.1',
    'description': 'An API client for EdgeScan',
    'long_description': None,
    'author': 'Tyler Fisher',
    'author_email': 'tylerfisher@tylerfisher.ca',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from edgescan.data.types.object import Object

IP = 'ip'
CIDR = 'cidr'
BLOCK = 'block'


@dataclass(frozen=True)
class LocationSpecifier(Object):
    id: int
    location: str
    location_type: str

    def is_ip(self) -> bool:
        return self.location_type == IP

    def is_ip_range(self) -> bool:
        return self.location_type == BLOCK

    def is_cidr(self) -> bool:
        return self.location_type == CIDR
